% Initial coding 10/01/08 for 3mb lib version 4.2

% Returns all animat states.

function as = ReadAnimatPopulationStateByAnimat(...
            fid,...             File to read from
            SParams,...         Scenario parameters set up in the 3mb by
                      ...       the user at run time.
            SVars,...           The selected output state variables that
                  ...           the script that called this script requests
            MaxAnimatCount, ... The maximum number of animat states to
                            ... collect.
            ANIMATSTATE)

    
as = [];

% Read in the animat states
a = 0;
numAnimats = SParams.totalNumAnimats;
if numAnimats > MaxAnimatCount
    numAnimats = MaxAnimatCount;
end
    
for m=1:numAnimats
    %display(sprintf('Extracting Animat: %05d', m));
    
    %if mod(m, 1000) == 0
    %    pause(20);
    %end
    
    % Check if a list of animats to display was passed in.  The call to
    % isempy() will return 0 (for false) if so.
    isEmpty = isempty(SVars.animatSel);
    if isEmpty == 0 % There is a list.
        isEmpty = isempty(find(SVars.animatSel == m));
        if isEmpty == 1 % Animat NOT found in the list
            skipBytes = SParams.regionSize.animatStates * ...
                SParams.numIterations; 
            fseek(fid, skipBytes, 'cof');
            continue; % continue the for-loop.
        end
    end

    a = a + 1;
    as = [as, ANIMATSTATE];
   
    for n=1:SParams.numIterationsSaved
        
        if n > 1
            n = n;
        end
        
        % Check if a list of iterations to display was passed in.  The call
        % to isempy() will return 0 (for false) if so.
        if 0 == isempty(SVars.iterationRange)
            if (n < SVars.iterationRange(1)) || (SVars.iterationRange(2)<n)
                fseek(fid, SParams.regionSize.animatStates, 'cof');
                continue; % continue the for-loop.
            end
        end
       
        % Animat ID
        if SParams.binOutConfig.animat.ID == 1
            if SVars.id == 1
                as(a).id = [as(a).id, fread(fid,1,'uint32')];
            else
                fseek(fid, 4, 'cof');
            end
        end

        % Current animat subID, if any.
        if SParams.binOutConfig.animat.subID
            if SVars.subID == 1
                as(a).subID = ...
                    [as(a).subID, fread(fid, 1, 'uint32')];
            else
                fseek(fid, 4, 'cof');
            end
        end        
        
        
        % Time of Day
        if SParams.binOutConfig.animat.timeOfDay == 1
            if SVars.clock == 1
                as(a).timeOfDay=[as(a).timeOfDay, fread(fid, 1, 'uint32')];
            else
                fseek(fid, 4, 'cof');
            end
        end

        % Coordinates
        if SParams.binOutConfig.animat.coordinate == 1
            if SVars.coords == 1
                as(a).lat = [as(a).lat, fread(fid, 1, 'float')];
                as(a).lon = [as(a).lon, fread(fid, 1, 'float')];
            else
                fseek(fid, 2*4, 'cof');
            end
        end

        % Depth
        if SParams.binOutConfig.animat.depth == 1
            if SVars.depth == 1
                as(a).depth = [as(a).depth, fread(fid, 1, 'float')];
            else
                fseek(fid, 4, 'cof');
            end
        end

        % Bearing
        if SParams.binOutConfig.animat.bearing == 1
            if SVars.bearing
                as(a).bearing = [as(a).bearing, fread(fid, 1, 'float')];
            else
                fseek(fid, 4, 'cof');
            end
        end
        
        % Dive Rate
        if SParams.binOutConfig.animat.diveRate == 1
            if SVars.diveRate
                as(a).diveRate = [as(a).diveRate, fread(fid, 1, 'float')];
            else
                fseek(fid, 4, 'cof');
            end
        end
        
        % Travel Rate
        if SParams.binOutConfig.animat.travelRate == 1
            if SVars.travelRate
                as(a).travelRate = ...
                    [as(a).travelRate, fread(fid, 1, 'float')];
            else
                fseek(fid, 4, 'cof');
            end
        end
        
        
        % Cummulative Acoustic Exposure
        if SParams.binOutConfig.animat.aeCmltve == 1
            if SVars.cExposure
                as(a).cExposure=[as(a).cExposure, fread(fid, 1, 'float')];
            else
                fseek(fid, 4, 'cof');
            end
        end
        
        % Instantaneous Acoustic Exposure
        if SParams.binOutConfig.animat.aeMoment == 1
            if SVars.iExposure
                as(a).iExposure=[as(a).iExposure, fread(fid, 1, 'float')];
            else
                fseek(fid, 4, 'cof');
            end
        end
        
        % Acoustic source angle
        if SParams.binOutConfig.animat.aeRelAngle == 1
            if SVars.relAngle == 1
                as(a).relAngle = [as(a).relAngle, fread(fid, 1, 'float')];
            else
                fseek(fid, 4, 'cof');
            end
        end
      
        % Acoustic Exposure Time Averting
        if  SParams.binOutConfig.animat.aeTimeAvrt == 1
            timeAverting = fread(fid, 1, 'uint32');
            if(timeAverting ~= 0)
                as(a).aeAverting = [as(a).aeAverting, 1];
            else
                as(a).aeAverting = [as(a).aeAverting, 0];
            end
            if SVars.secAvert == 1
                as(a).secAvert = [as(a).secAvert, timeAverting];
            end
        end
        
        % Bathymetry Depth at animat location.
        if SParams.binOutConfig.animat.bathyDepth == 1
            if SVars.bathyDepth == 1
                as(a).bathyDepth = ...
                    [as(a).bathyDepth, fread(fid, 1, 'float')];
            else
                fseek(fid, 4, 'cof');
            end
        end
        
        % Salinity at animat location
        if SParams.binOutConfig.animat.salinity == 1
            if SVars.salinity == 1
                as(a).salinity = [as(a).salinity, fread(fid, 1, 'float')];
            else
                fseek(fid, 4, 'cof');
            end
        end
        
        %  Temperature at animat location.
        if SParams.binOutConfig.animat.temperature == 1
            if SVars.temperature == 1
                as(a).temperature= [as(a).temperature, fread(fid, 1, 'float')];
            else
                fseek(fid, 4, 'cof');
            end
        end
        
        % Read in packed data.
        if SParams.binOutConfig.animat.packedData
            packedData = fread(fid, 1, 'uint32');
            unpackedData = UnpackData(packedData);

            % Behavior State (from the unpacked data)
            if SVars.behavior == 1
                as(a).behavior = [as(a).behavior, unpackedData.behavior];
            end

            % Acoustice Exposure Threshold flag (from the unpacked data)
            if SVars.thrshFlag == 1
                as(a).thrshFlag=[as(a).thrshFlag, unpackedData.threasholdFlag];
            end

            % Dive Activity (from the unpacked data)
            %0=SurfaceInterval, 1=bottomfollowing, 2=ascending, 3=descending,
            %4=ascendingReversal, 5=descendingReversal, 6=beached.
            if SVars.diveActivity == 1
                as(a).diveActivity=[as(a).diveActivity, unpackedData.activity];
            end

            % Behavior modifiers, 5 bits (numbered 0~4) (from the unpacked
            % data).  Bits indicate which behavior modifier is active.
            %    1=active
            %    0=not active.
            %
            % Bit 0 - acoustic aversion, bit 1 - pod follow, bit 2 shore
            % follow, bit 3 environmental attractor depth, bit 4 environmental
            % attractor temperature
            if SVars.overrideBehavior == 1
                as(a).overrideBehavior = ...
                    [as(a).overrideBehavior, unpackedData.activity];
            end

            if SVars.offscreenReset == 1
                as(a).offscreenReset = ...
                    [as(a).offscreenReset, unpackedData.offscreenReset];
            end
        end
        
        %  Target depth
        if SParams.binOutConfig.animat.targetDepth
            if SVars.targetDepth == 1
                as(a).targetDepth =[as(a).targetDepth, fread(fid, 1, 'float')];
            else
                fseek(fid, 4, 'cof');
            end
        end

        %  Calculated Depth
        if SParams.binOutConfig.animat.calcDepth
            if SVars.calculatedDepth == 1
                as(a).calculatedDepth = ...
                    [as(a).calculatedDepth, fread(fid, 1, 'float')];
            else
                fseek(fid, 4, 'cof');
            end
        end

        % Net X and Y distance traveled
        if SParams.binOutConfig.animat.xyDistance
            if SVars.xyDistance == 1
                as(a).xDistance = ...
                    [as(a).xDistance, fread(fid, 1, 'float')];
                as(a).yDistance = ...
                    [as(a).yDistance, fread(fid, 1, 'float')];                
            else
                fseek(fid, 8, 'cof');
            end
            
        end
        
        % Current animat risk, if any.
        if SParams.binOutConfig.animat.riskValue
            if SVars.riskValue == 1
                as(a).riskValue = ...
                    [as(a).riskValue, fread(fid, 1, 'float')];
            else
                fseek(fid, 4, 'cof');
            end
        end
        
    end
end

return;