function SMB_SPECIES_DENSITY_POLYGONS = ReadSmbPolygons(fid)

SMB_SPECIES_DENSITY_POLYGONS = getZeroedSmbPolygonToFileStruct;

SMB_SPECIES_DENSITY_POLYGONS.speciesCommonNameString = fread(fid, 32, 'uint8=>char')';
SMB_SPECIES_DENSITY_POLYGONS.speciesSciNameString = fread(fid, 32, 'uint8=>char')';
SMB_SPECIES_DENSITY_POLYGONS.seasonString = fread(fid, 16, 'uint8=>char')';
SMB_SPECIES_DENSITY_POLYGONS.polygonClusterCountString = fread(fid, 16, 'uint8=>char')';
fseek(fid, 4, 'cof'); % skip over a 4 byte buffer.
SMB_SPECIES_DENSITY_POLYGONS.polygonClusterCount = fread(fid, 1, 'int');
fseek(fid, 8, 'cof'); % skip over a 8 byte pointer buffer.

if 0 == SMB_SPECIES_DENSITY_POLYGONS.polygonClusterCount
    return;
end

% Preallocate memory for the cluster array.
SMB_SPECIES_DENSITY_POLYGONS.clusterArray = ...
    GetSmbPolygonClusterFileStructArray(SMB_SPECIES_DENSITY_POLYGONS.polygonClusterCount);

% Read in the cluster array.
for n=1:SMB_SPECIES_DENSITY_POLYGONS.polygonClusterCount
    SMB_SPECIES_DENSITY_POLYGONS.clusterArray(n) = ReadSmbPolygonClusterFileStruct(fid);
end

% Read in the inner box
SMB_SPECIES_DENSITY_POLYGONS.innerOpsBox = ReadSmbOpsArea(fid);

% Read in the outer box.
SMB_SPECIES_DENSITY_POLYGONS.outerOpsBox = ReadSmbOpsArea(fid);

end

function SMB_SPECIES_DENSITY_POLYGONS = getZeroedSmbPolygonToFileStruct
    SMB_SPECIES_DENSITY_POLYGONS.speciesCommonNameString = '';
    SMB_SPECIES_DENSITY_POLYGONS.speciesSciNameString = '';
    SMB_SPECIES_DENSITY_POLYGONS.seasonString = '';
    SMB_SPECIES_DENSITY_POLYGONS.polygonClusterCountString = '';
    SMB_SPECIES_DENSITY_POLYGONS.polygonClusterCount = 0;
    SMB_SPECIES_DENSITY_POLYGONS.clusterArray = '';
    SMB_SPECIES_DENSITY_POLYGONS.innerOpsBox = '';
    SMB_SPECIES_DENSITY_POLYGONS.outerOpsBox = '';
end




%%
function POLYGON_CLUSTER_FILE = ReadSmbPolygonClusterFileStruct(fid)
POLYGON_CLUSTER_FILE = GetSmbPolygonClusterFileStructArray(1);
POLYGON_CLUSTER_FILE.string = fread(fid, 48, 'uint8=>char')';

POLYGON_CLUSTER_FILE.clusterNumber = fread(fid, 1, 'int');
POLYGON_CLUSTER_FILE.srcFid = fread(fid, 1, 'int');
POLYGON_CLUSTER_FILE.polygonCount = fread(fid, 1, 'int');
fseek(fid, 4, 'cof'); % skip over a 4 byte buffer.

POLYGON_CLUSTER_FILE.density = fread(fid, 1, 'double');
POLYGON_CLUSTER_FILE.area = fread(fid, 1, 'double');

POLYGON_CLUSTER_FILE.areaCalc = fread(fid, 1, 'double');
POLYGON_CLUSTER_FILE.depth = fread(fid, 1, 'double');

POLYGON_CLUSTER_FILE.maxLat = fread(fid, 1, 'double');
POLYGON_CLUSTER_FILE.maxLon = fread(fid, 1, 'double');

POLYGON_CLUSTER_FILE.minLat = fread(fid, 1, 'double');
POLYGON_CLUSTER_FILE.minLon = fread(fid, 1, 'double');
fseek(fid, 16, 'cof'); % skip over a 16 bytes of pointer and buffer.

POLYGON_CLUSTER_FILE.polygonArray = ...
    GetSmbPolygonFileHeaderStructArray(POLYGON_CLUSTER_FILE.polygonCount);

for n=1:POLYGON_CLUSTER_FILE.polygonCount
    POLYGON_CLUSTER_FILE.polygonArray(n) = ...
        ReadSmbPolygonFileHeaderStructPolygons(fid);
end
end


%%
function POLYGON_CLUSTER_FILE = GetSmbPolygonClusterFileStructArray(initSize)
POLYGON_CLUSTER_FILE = struct('string', 0, 'clusterNumber', 0, 'srcFid', 0, ...
    'polygonCount', 0, 'density', 0, 'area', 0, 'areaCalc', 0, 'depth', 0, 'maxLat', 0, ...
    'maxLon', 0, 'minLat', 0, 'minLon', 0, 'polygonArray', 0);
POLYGON_CLUSTER_FILE(initSize).clusterNumber = 0;
end

%%
function POLYGON_FILE = ReadSmbPolygonFileHeaderStructPolygons(fid)
POLYGON_FILE = GetSmbPolygonFileHeaderStructArray(1);
POLYGON_FILE.polygonString = fread(fid, 32, 'uint8=>char')';
POLYGON_FILE.polygonNumber = fread(fid, 1, 'int');
POLYGON_FILE.vertexCount = fread(fid, 1, 'int');
fseek(fid, 8, 'cof'); % skip over a 16 bytes of pointer and buffer.
POLYGON_FILE.vertices = ReadVertices(fid, POLYGON_FILE.vertexCount);
end

%%
function POLYGON_FILE = GetSmbPolygonFileHeaderStructArray(initSize)
POLYGON_FILE = struct ('polygonString', 0, 'polygonNumber', 0, 'vertexCount', 0, ...
    'vertices', 0);
POLYGON_FILE(initSize).polygonNumber = 0;
end

%%
function OPS_AREA_FILE = ReadSmbOpsArea(fid)

OPS_AREA_FILE.title =  fread(fid, 32, 'uint8=>char')';
OPS_AREA_FILE.name =  fread(fid, 32, 'uint8=>char')';
OPS_AREA_FILE.layer = fread(fid, 32, 'uint8=>char')';
OPS_AREA_FILE.perimeter = fread(fid, 32, 'uint8=>char')';
OPS_AREA_FILE.enclosedArea = fread(fid, 32, 'uint8=>char')';
OPS_AREA_FILE.maxLat = fread(fid, 1, 'double')';
OPS_AREA_FILE.maxLon = fread(fid, 1, 'double')';
OPS_AREA_FILE.minLat = fread(fid, 1, 'double')';
OPS_AREA_FILE.minLon = fread(fid, 1, 'double')';
fseek(fid, 4, 'cof'); % skip over a 4 byte buffer.
OPS_AREA_FILE.vertexCount = fread(fid, 1, 'int')';
fseek(fid, 8, 'cof'); % skip over pointer.

OPS_AREA_FILE.vertices = ReadVertices(fid, OPS_AREA_FILE.vertexCount);


% 
% OPS_AREA_FILE.polygon = ...
%     GetSmbPolygonFileHeaderStructArray(OPS_AREA_FILE.vertexCount);
% 
% for n=1:OPS_AREA_FILE.vertexCount
%     OPS_AREA_FILE.polygon(n) = ReadSmbPolygonFileHeaderStructPolygons(fid);
% end

end

